const config = require('config')
const { get } = require('lodash')

const unsubscribeSlug = config.get('unsubscribe.url')
const resetPath = config.get('invite-reset-password.url')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    reviewer,
    UserModel,
    collection,
    isHE = false,
    isNewUser = false,
    isAccepted = false,
    isCanceled = false,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const { title } = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `the manuscript titled "${title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.firstName}`

    const handlingEditor = get(collection, 'handlingEditor')
    const userHelper = new User({ UserModel })
    const { firstName, lastName } = await userHelper.getEditorInChief()
    const eicName = `${firstName} ${lastName}`
    const subjectBaseText = isCanceled
      ? `${collection.customId}: Reviewer `
      : `${collection.customId}: Manuscript `

    const email = new Email({
      type: 'user',
      content: {
        signatureName: handlingEditor.name,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
      },
    })

    if (isHE) {
      sendReviewerEmail({
        email,
        baseUrl,
        reviewer,
        titleText,
        isCanceled,
        subjectBaseText,
      })
    } else {
      sendHandlingEditorEmail({
        email,
        eicName,
        titleText,
        handlingEditor,
        targetUserName: `${reviewer.firstName} ${reviewer.lastName}`,
        emailType:
          isAccepted === true ? 'reviewer-accepted' : 'reviewer-declined',
      })
    }

    if (isNewUser) {
      sendNewUserEmail({ email, baseUrl, user: reviewer, titleText })
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  eicName,
  baseUrl,
  titleText,
  emailType,
  handlingEditor,
  targetUserName,
  subjectBaseText,
}) => {
  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }

  email.content.subject = `${subjectBaseText} Reviews`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })
  email.content.signatureName = eicName

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
      expectedDate: services.getExpectedDate({
        timestamp: Date.now(),
        daysExpected: 14,
      }),
      targetUserName,
    }),
  })
  email.sendEmail({ html, text })
}

const sendReviewerEmail = async ({
  email,
  baseUrl,
  reviewer,
  titleText,
  isCanceled,
  subjectBaseText,
}) => {
  email.content.subject = isCanceled
    ? `${subjectBaseText} Unassigned`
    : `${subjectBaseText} Review`
  const emailType = isCanceled
    ? 'reviewer-cancel-invitation'
    : 'reviewer-thank-you'

  email.toUser = {
    email: reviewer.email,
    name: `${reviewer.firstName} ${reviewer.lastName}`,
  }

  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: reviewer.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}

const sendNewUserEmail = ({ email, baseUrl, user, titleText }) => {
  email.toUser = {
    email: user.email,
    name: user.name,
  }

  email.content.subject = `Confirm Your Account`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: user.id,
  })
  email.content.ctaLink = services.createUrl(baseUrl, resetPath, {
    email: user.email,
    token: user.passwordResetToken,
    firstName: user.firstName,
    lastName: user.lastName,
    affiliation: user.affiliation,
    title: user.title,
  })
  email.content.ctaText = 'CONFIRM ACCOUNT'

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType: 'reviewer-new-account',
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}
